/*
	$Header$
	$Log$ 
 */

package COM.webconcepts.net;

import java.net.*;
import java.io.*;

/** 
  * An HttpProxyConnection is used by an applet that wants to fetch a URL that is
  * outside of the local Web server. It relies on an instance of HttpProxyServer running
  * on a known port (usually 4005) on the server where the applet is hosted.
  * 
  * @author Kyle Downey
  * @author Copyright 1997, WebConcepts, LLC, http://www.webconcepts.com
  * @author This code is free for use under the terms of the GNU General Public License
  * @author http://www.fsf.org
  *
  * @see COM.webconcepts.net.HttpProxyServer
  */
public class HttpProxyConnection {
	// the name of the proxying host
	String proxyHost;

	// the port that HttpProxyServer is running on
	int proxyPort;

	/** 
	  * creates a new proxy connection to the named host using a non-standard port
	  * @param proxyHost the host on which the proxy server is running
	  * @param proxuPort the port on which the proxy server is listening
	  */
	public HttpProxyConnection(String proxyHost, int proxyPort) {
		this.proxyHost = proxyHost;
		this.proxyPort = proxyPort;
	}

	/**
	  * creates a new proxy connection to the name host using the standard port
	  * (usually 4005)
	  * @param proxyHost the host on which the proxy server is running
	  */
	public HttpProxyConnection(String proxyHost) {
		this.proxyHost = proxyHost;
		this.proxyPort = COM.webconcepts.net.HttpProxyServer.DEFAULT_HTTP_PROXY_PORT;
	}

	/**
	  * loads a remote URL into a byte array
	  * @param proxyURL the URL to load
	  * @exception Exception if there is a network error when talking to the server
	  * @return a byte array with the remote URL's contents
	  */
	public byte[] fetch(URL proxyURL) throws Exception {
		System.out.println("fetching " + proxyURL.toString() + " using proxy at " +
			proxyHost + ":" + proxyPort);
		Socket sock = new Socket(proxyHost, proxyPort);

		InputStream in = sock.getInputStream();
		DataInputStream urlData = new DataInputStream(in);
		OutputStream out = sock.getOutputStream();
		PrintStream cmdStream = new PrintStream(out);

		// fetch past the first line
		urlData.readLine();

		// send a PROXY command
		cmdStream.println("PROXY " + proxyURL.toString());

		// get the status
		String cmdResult = urlData.readLine();
		System.out.println(cmdResult);
		if (cmdResult.regionMatches(0, "+OK", 0, 3)) {
			// the next line is 
			// Content-length: xxx 
			// so the number is between character 17 and the end
			int contentLength = Integer.parseInt(urlData.readLine().substring(17));
			byte[] dataBuffer = new byte[contentLength];
			byte[] chunk = new byte[512];
			int count;
			int bytesRead = 0;

			// fetch the whole piece of data into a byte array
			while (bytesRead <= contentLength) {
				count = urlData.read(chunk);
				bytesRead += count;
		        byte[] t = new byte[dataBuffer.length + count];
				System.arraycopy(dataBuffer, 0, t, 0, dataBuffer.length);
				System.arraycopy(chunk, 0, t, dataBuffer.length, count);
				dataBuffer = t;
			}

			// send a QUIT command
			cmdStream.println("QUIT");
			sock.close();
	
			return dataBuffer;
		}
		else {
			sock.close();
			throw new Exception("proxy error: " + cmdResult);
		}

	}
}