/*
	$Header$
	$Log$ 
 */

package COM.webconcepts.net;

import java.net.*;
import java.io.*;
import java.util.*;

/** 
  * An HttpProxyServer is a command-line daemon based on the MultiThreadServer framework.
  * It accepts all of the default behaviors from that framework and implements a simple
  * main() method for command-line launching of the service. All of the protocol handling
  * and real work is done by HttpProxyServiceThread.
  * 
  * @author Kyle Downey
  * @author Copyright 1997, WebConcepts, LLC, http://www.webconcepts.com
  * @author This code is free for use under the terms of the GNU General Public License
  * @author http://www.fsf.org
  *
  * @see COM.webconceots.net.HttpProxyServiceThread
  * @see COM.webconcepts.net.MultiThreadServer
  */
public class HttpProxyServer extends MultiThreadServer {
	public static final int DEFAULT_HTTP_PROXY_PORT = 4005;

	/** 
	  * creates a new proxy server running on the name port
	  * @param servicePort the listener port
	  * @exception if there is any trouble initializing the MultiThreadServer
	  */
	HttpProxyServer(int servicePort) throws Exception  {
		super(Class.forName("COM.webconcepts.net.HttpProxyServiceThread"), servicePort);
	}

	/**
	  * launches the proxy service
	  * @param args the command-line arguments: either none or an integer naming the
	  * port to listen on for proxy request
	  */
	public static void main(String[] args) {
		try {
			int servicePort;
			if (args.length == 1) {
				try {
					servicePort = Integer.parseInt(args[1]);
				}
				catch (Exception e) {
					System.out.println("Usage: java COM.webconcepts.net.HttpProxyServer [portNum]");
					System.exit(1);
				}
			}
			else {
				servicePort = DEFAULT_HTTP_PROXY_PORT;
			}

			System.out.println("Applet HTTP Proxy Server 1.0, (C) WebConcepts, LLC, 1997");
			HttpProxyServer proxyServer = new HttpProxyServer(4005);
			proxyServer.init();
		}
		catch (Exception e) {
			System.out.println("proxy server caught an exception");
			e.printStackTrace();
		}
	}
}