/*
	$Header$
	$Log$ 
 */

package COM.webconcepts.net;

import java.net.*;
import java.io.*;
import java.util.*;

/** 
  * An HttpProxyServiceThread mediates communication between a network client who
  * wants a copy of a URL and the remote server. It uses the MultiThreadServer framework
  * for most of the work, and simply defines service() to implement the proxying protocol.
  * The protocol has only two commands: PROXY <i>url</i> and QUIT. PROXY returns a 
  * Content-length: xxx line with the length of the URL contents followed by the conents
  * themselves. QUIT terminates the network connection. All commands result in either
  * a +OK or +ERR return code for the client, as in the POP3 protocol.
  * 
  * @author Kyle Downey
  * @author Copyright 1997, WebConcepts, LLC, http://www.webconcepts.com
  * @author This code is free for use under the terms of the GNU General Public License
  * @author http://www.fsf.org
  *
  * @see COM.webconcepts.net.HttpProxyServer
  * @see COM.webconcepts.net.ServiceThread
  * @see COM.webconcepts.net.MultiThreadServer
  */
public class HttpProxyServiceThread extends ServiceThread {
	/**
	  * service() is called by the run() method of ServiceThread whenever it
	  * accept()s a new client. It expects the client to communicate using the simple
	  * proxying protocol outlined above. Error-checking is very limited in this
	  * demonstration version.
	  * @param in an input stream from the client
	  * @param out an output stream directed to the client
	  */
	public void service(InputStream in, OutputStream out) throws Exception {
		PrintStream os = new PrintStream(out);
		DataInputStream is = new DataInputStream(in);

		os.println("Applet HTTP Proxy Server 1.0, (C) WebConcepts, LLC, 1997");

		while (true) {
			String commandLine = is.readLine();
			if (commandLine.regionMatches(true, 0, "PROXY ", 0, 6)) {
				String proxyUrlString = commandLine.substring(6);
				System.out.println("proxying to " + proxyUrlString);

				// fetch the file specified in the URL and redirect it to the client
				URL proxyURL = new URL(proxyUrlString);
				InputStream urlStream = proxyURL.openStream();

				// read the stream into buffer
				byte[] buffer = new byte[0];
				byte[] chunk = new byte[4096];
				int count;
				while ((count = urlStream.read(chunk)) >= 0) {
					byte [] t = new byte[buffer.length + count];
					System.arraycopy(buffer, 0, t, 0, buffer.length);
					System.arraycopy(chunk, 0, t, buffer.length, count);
					buffer = t;
				}

				// write the buffer to the output stream
				os.println("+OK URL fetched; data to follow");
				os.println("Content-length: " + buffer.length);
				os.write(buffer);
			}
			else if (commandLine.regionMatches(true, 0, "QUIT", 0, 4)) {
				os.println("+OK logging out of proxy server");
				return;
			}
			else {
				os.println("+ERR unknown directive");
				continue;
			}
		}
	}
}