/*
	$Header$
	$Log$ 
 */

package COM.webconcepts.net;

import java.net.*;
import java.io.*;
import java.util.*;

/** 
  * The MultiThreadServer framework allows you to develop the client-handling
  * code separately from the core server systems. Clients are handled by objects
  * that extend this class. ServiceThread negotiates the initial connection with the
  * client, and then passes control to the service() method, which is a no-op in
  * this class.
  *
  * @author Kyle Downey
  * @author Copyright 1997, WebConcepts, LLC, http://www.webconcepts.com
  * @author This code is free for use under the terms of the GNU General Public License
  * @author http://www.fsf.org
  *
  * @see COM.webconcepts.net.MultiThreadServer
  */
public class ServiceThread extends Thread {
	MultiThreadServer server;

	/** 
	  * sets a reference back to the host server object
	  * @param server the host server
	  */
	public void setServer(MultiThreadServer server) {
		this.server = server;
	}

	/**
	  * gets the host server for this thread
	  * @return the host server reference
	  */
	public MultiThreadServer getServer() {
		return server;
	}

	/**
	  * run() loops continuously until the thread dies or is shut down.
	  * It accept()s clients as they make requests and dispatches the service()
	  * method to handle them
	  */
	public void run() {
		Socket s = null;
		InputStream in = null;
		OutputStream out = null;

		try {
			while(true) {
				// connect up the sockets and streams
				s = server.getServerSocket().accept();
				System.out.println("thread " + Thread.currentThread().getName() + " accepting");

				in = s.getInputStream();
				out = s.getOutputStream();

				// communicate with the remote client
				service(in, out);

				// clean up
				out.close();
				in.close();
			}
		}
		catch (Exception e) {
			handleServiceException(e);
		}
	}

	/** 
	  * This empty method is called whenever a client connects to this service thread;
	  * objects that implement a particular network protocol should define this method
	  * to communicate with the client via the streams.
	  * @param in the client input stream
	  * @param out a stream that writes back to the client
	  * @exception if there are any unrecoverable network or I/O errors
	  */
	public void service(InputStream in, OutputStream out) throws Exception {
		;
	}

	/**
	  * This method is called in case there are any exceptions while the thread is
	  * running
	  * @param e the exception thrown
	  */
	public void handleServiceException(Exception e) {
		e.printStackTrace();
	}
}